# XCHK_AFM.afm (DOS/GAWK version)
#    usage: gawk -f xchk_afm.awk
#
# Check group of AFM files for different layout and each afm/pfb pair 
# for consistency (the same layout). AFM/PFB files should be present
# in the current directory.
#
# Note: T1DISASM shoud be present in your system while running this script!
#
# TP (april 5, 1996)
#

# CMP_FILES: compare afm/pfb layout
function cmp_files() {
   for (i=1; i<=pfb_char_num; i++)
        if ( pfb_char[i] != char[i] ) {return (1);  }
   return (0)
}

# CHK_PFB: extract characters from pfb file
function chk_pfb(fn, xc) {
   gsub (/\.AFM/,"\.PFB", fn); # we are checking pfb file now;
   print fn, "found" >> stdio;
   ## system ("arj x.....") ##
   
   system ("t1disasm " fn "\> disasm.tmp") #
   scan = 0;
   xc = 0;

   while ( (getline pfb < "disasm.tmp" )  > 0) {
      
      if (pfb ~ /\/CharStrings/) {
         scan = 1; 
         print "Scanning pfb..." >> stdio;
         getline pfb < "disasm.tmp";
      } 
      else {
         if ( (scan == 1) && (pfb ~ /readonly put put/) ) {
            scan = 0; 
            print "\ndone..." >> stdio;; 
            close ("disasm.tmp");
            return (xc);
         }
      }

      if (scan && pfb ~ /^\//) {##
          xc++;
          gsub(/\//,"", pfb);
          split(pfb, tmp, " ");
          pfb_char[xc] = tmp[1];
          printf "" >> stdio;
      }
   }

   # in case of any strange error we put:
   close ("disasm.tmp")
   return(-1);
}

# MAIN routine:
BEGIN { stdio = "con" # stdio for DOS


   while (("dir/b *.afm" | getline  fname ) > 0) {
      scan = 0;    charno = 0;
      print "\n" fname, "found" >> stdio;
      files++;

      pfb_char_num = chk_pfb(fname);

      print "Chars defined in pfb file:", pfb_char_num >> stdio;
      

      while ( (getline < fname )  > 0) {

         if ($1 == "StartCharMetrics") {scan = 1; getline < fname } else 
            {if ($1 ~ "EndCharMetrics") {scan = 0; break }}

         if (scan) {# ###
            charno++;
            char[charno] = $8;
         }
      }
     
      print "Chars defined in afm file:", charno >> stdio;
      if ( cmp_files() ) { print " Afm/pfb files are inconsistent!" >> stdio; }

      # chk all layouts --------------------
      maxcharno = charno 

      if (files==1) {# scanning first file --
         layout_num = 1;
         layouts[0,layout_num] = fname;  # store file name 
         for (i=1; i <= charno; i++) { layouts[i,layout_num] = char[i] }
      } 
      else {

         for (i=1; i<=layout_num; i++) {
             diff = 0; 
             for (j=1; j <= charno; j++) {
                 if (layouts[j,i] != char[j]) { diff++ }
                    # ##print layouts[j,i], char[j];
             }

             
             if (diff == 0) {# found identical
                layouts[0,i] = layouts[0,i] ";" fname; # store file name 
                break; 
             }
         }


         if (diff) {## another layout!
            print "New layout found!" >> stdio;
            layout_num++;
            for (i=1; i <= charno; i++) { 
               layouts[i,layout_num] = char[i] 
            }

            layouts[0,layout_num] = fname; # store file name 
         }   

      }

     close (fname)

   }
   close ("dir/b *.afm");
 
   # Print characters in all layouts

   printf "Scanned %d files\n", files >> stdio;
   printf "Found %s layout(s)\n", layout_num >> stdio;
   printf "inspect chk-afm.log file for details\n", layout_num >> stdio;

   # once again to log file (possible)

   printf "Scanned %d files\n", files;
   printf "Found %s layout(s)\n", layout_num;
   printf " detailed statistics is as follows:\n";

   for (i=1; i<=layout_num; i++) {print " -- layout", i, "in: ", 
        layouts[0,i];}

   for (i=1; i<=layout_num; i++) {
      printf "\nLayout %d\n======\n", i;
      clen = 0;

      for (j=1; j<=maxcharno; j++) {
           printf "%s, ", layouts[j,i];
           clen += (2 + length(layouts[j,i]));
           if (clen > 60) { clen = 0; printf "\n"; }
      }
      printf "\n";
   }
   
}

